import { Metadata, Http, Config } from '@zeedhi/core';
import { ITabsBeforeChangeEventParam, Modal, ModalService, LoadingService, DialogService, FileInput } from '@zeedhi/common';
import { DialogServiceCustom } from '@/services/DialogServiceCustom';
import { AlertServiceCustom } from '@/services/AlertServiceCustom';
import { InstanceService } from '@/services/InstanceService';
import Messages from '@/services/Messages';
import { IterableComponentRender, TekRestDatasource, ITekGridLayout, TekGrid } from '@zeedhi/teknisa-components-common';
import { FileUpload } from '@/services/FileUpload';
import { Utilities } from '@/services/Utilities';
import { ZdList } from '@zeedhi/vuetify';
import { ActivitiesDemandManagement } from '@/services/ActivitiesDemandManagement';
import Store from '../../store';

export class DemandManagementController {
	public defaultTimeoutTimer: any;

	public callDescription = '';

	public baseUrl = '';

	public callType = '';

	public callRelationship = '';

	public callID = '';

	public isLoadingDefault = false;

	public showDemandManagementForm = false;

	public showFormEditIssue = false;

	private userAuth: any;

	private FIELDS_NOT_INTEGRATION = [
		'ID',
		'STATUS',
		'Fechado',
		'TASK_STATUS',
		'CDFASETRAM',
		'NRCALLLABEL',
		'CALL_WITHOUT_LABELS_CHECK',
		'ALREADY_READ',
	];

	private rowSpec: any;

	get notIsVisibleIssueActions() {
		return !this.isVisibleIssueActions;
	}

	private modalAddCall!: Modal;

	private modalForwardCall!: Modal;

	private modalListSpecification!: Modal;

	private modalListSpecificationIssue!: Modal;

	private modalIssueActions!: Modal;

	private modalAddComments!: Modal;

	private modalUploadFile!: Modal;

	// START SHARED VARIABLES ADD/EDIT ---
	private typeIsI = false;

	private typeIsS = false;

	private callIsClosed = null;

	public clientNOTSelected = true;

	public clientNOTSelectedAdd = true;

	public productNOTSelected = true;

	/**
	 * Variaveis e métodos de controle de "disable"
	 * dos selects do form de editar issue
	 */
	public clientEditINotSelected = true;

	public productEditINotSelected = true;

	public screenNameEditINotSelected = false;

	public versionEditINotSelected = true;

	public getCallRelationship() {
		return this.callRelationship;
	}

	public get disableProductEditI() {
		return this.clientEditINotSelected || this.isCallTypeS;
	}

	public get disableCategoryEditI() {
		return this.productEditINotSelected;
	}

	public get disableVersionEditI() {
		return this.screenNameEditINotSelected;
	}

	public get disableDevVersionEditI() {
		return this.screenNameEditINotSelected;
	}

	/** fim */

	get isCallTypeS() {
		return this.callType === 'S';
	}

    get isVisibleDemandManagementGrid() {
		return !this.showDemandManagementForm;
    }

	public get disableProduct() {
		return this.clientNOTSelected;
	}

	public get disableProductAdd() {
		if (this.clientNOTSelectedAdd) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public get disableCategory() {
		return this.productNOTSelected;
	}

	public productNOTSelectedAdd = true;

	public get disableCategoryAdd() {
		if (this.productNOTSelectedAdd) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public screenNameNOTSelected = true;

	public get disableScreenNameAdd() {
		const productEdit = InstanceService.getInstanceZdSelect('productEdit');
		return !Object.keys(productEdit.selectedData).length;
	}

	public get disableVersionI() {
		if (this.screenNameNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public productTypeNOTSelected = true;

	public get disableVersionS() {
		return this.productTypeNOTSelected;
	}

	public versionNOTSelected = true;

	public get disableRelease() {
		return this.versionNOTSelected;
	}

	public releaseNOTSelected = true;

	public get disablePatch() {
		if (this.releaseNOTSelected) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public get disableUseLicense() {
		return this.clientNOTSelected;
	}
	// END SHARED VARIABLES ADD/EDIT ---

	get showTabIssue() {
		return this.showDemandManagementForm && this.callType === 'I';
	}

	get showTabSolicitation() {
		return this.showDemandManagementForm && this.callType === 'S';
	}

	public async onMountedGeneralCard() {
		await window.PM_PROMISE_REQUEST_USER_AUTH;
		this.baseUrl = Config.env.endPoint;
		this.userAuth = Store.getters['User/getUser'];
	}

	public changeDemandTabs(params: any) {
		if (params.component.activeTab === 0) {
			setTimeout(async () => {
				this.preSetFilterCall();
			}, 100);
		} else if (params.component.activeTab === 1) {
			setTimeout(async () => {
				this.preSetFilterCallCLosed();
			}, 100);
		}
	}

	public removeFieldsNotIntegration(dynamicFilter:object, dynamicFilterOrigin:any) {
		const keys = Object.keys(dynamicFilter);
		const keysOrigin = Object.keys(dynamicFilterOrigin);
		const values = Object.values(dynamicFilter);
		const difference = keysOrigin.filter((x) => !keys.includes(x));
		difference.forEach((fieldName: string) => {
			if (!this.FIELDS_NOT_INTEGRATION.includes(fieldName)) {
				delete dynamicFilterOrigin[fieldName];
			}
		});
		keys.forEach((fieldName: string, index:number) => {
			if (!this.FIELDS_NOT_INTEGRATION.includes(fieldName)) {
				dynamicFilterOrigin[fieldName] = values[index];
			}
		});
		console.log(dynamicFilterOrigin);
	}

	public preSetFilterCallCLosed() {
		try {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const callGridInstanceClosed = InstanceService.getInstanceTekGrid('gridDemandManagementClosed');
			this.setFilterCallCLosed(callGridInstance, callGridInstanceClosed);
		} catch (e) {
			setTimeout(async () => {
				this.preSetFilterCallCLosed();
			}, 100);
		}
	}

	public preSetFilterCall() {
		try {
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const callGridInstanceClosed = InstanceService.getInstanceTekGrid('gridDemandManagementClosed');
			this.setFilterCall(callGridInstance, callGridInstanceClosed);
		} catch (e) {
			setTimeout(async () => {
				this.preSetFilterCall();
			}, 100);
		}
	}

	public async onMountedDemandManagementGridClosed() {
		const currentUser = await Http.get('findCurrentUser');
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagementClosed');
		if (callGridInstance.datasource instanceof TekRestDatasource) {
			const dynamicFilter = {
				REPORTED_BY: [
					{
						operation: 'IN',
						relation: 'AND',
						value: [currentUser.data.CDOPERADOR],
					},
				],
				STATUS: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['C'],
					},
				],
			};
			const defaultLayout: ITekGridLayout = {
				name: 'Padão',
				dynamicFilter,
			};
			callGridInstance.layoutOptions?.updateDefaultLayout(defaultLayout);
		}
	}

	public setFilterCallCLosed(callGridInstance:TekGrid, callGridInstanceClosed:TekGrid) {
		if (callGridInstanceClosed
			&& callGridInstance.datasource instanceof TekRestDatasource
			&& callGridInstanceClosed.datasource instanceof TekRestDatasource) {
			if (callGridInstanceClosed.layoutOptions) {
				callGridInstanceClosed.layoutOptions.currentLayoutName = 'Filtro compartinhado';
				this.onMountedDemandManagementGridClosed();
			}
			this.removeFieldsNotIntegration(callGridInstance.datasource.dynamicFilter, callGridInstanceClosed.datasource.dynamicFilter);
			callGridInstanceClosed.datasource.dynamicFilter.STATUS = [{ operation: 'IN', relation: 'AND', value: ['C'] }];
			callGridInstanceClosed.datasource.get();
		}
	}

	public setFilterCall(callGridInstance:TekGrid, callGridInstanceClosed:TekGrid) {
		if (callGridInstanceClosed
			&& callGridInstance.datasource instanceof TekRestDatasource
			&& callGridInstanceClosed.datasource instanceof TekRestDatasource) {
			this.removeFieldsNotIntegration(callGridInstanceClosed.datasource.dynamicFilter, callGridInstance.datasource.dynamicFilter);
			callGridInstance.datasource.get();
		}
	}

	public async onMountedDemandManagementGrid() {
		this.modalAddCall = await ModalService.createFromJson('demandManagement/modal/addCallDemandManagementModal', true);
		this.modalForwardCall = await ModalService.createFromJson('demandManagement/modal/forwardCallDemandManagementModal', true);
		this.modalListSpecification = await ModalService.createFromJson('demandManagement/modal/listSpecificationIssueModal', true);
		this.modalListSpecificationIssue = await ModalService.createFromJson('demandManagement/modal/listSpecificationIssueModalIssue', true);
		const currentUser = await Http.get('findCurrentUser');
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		if (callGridInstance.datasource instanceof TekRestDatasource) {
			const dynamicFilter = {
				REPORTED_BY: [
					{
						operation: 'IN',
						relation: 'AND',
						value: [currentUser.data.CDOPERADOR],
					},
				],
				STATUS: [
					{
						operation: 'IN',
						relation: 'AND',
						value: ['O', 'P', 'W'],
					},
				],
			};
			const defaultLayout: ITekGridLayout = {
				name: 'Padão',
				dynamicFilter,
			};
			callGridInstance.layoutOptions?.updateDefaultLayout(defaultLayout);
		}
	}

	public onDestroyDemandManagementGrid() {
		this.modalAddCall?.destroy();
		this.modalForwardCall?.destroy();
	}

	// START CALL GRID ---
	public publicComment = false;

	public checkForwardCall(params: any) {
		return params.row.FORWARD_CALL_GRID === 'S';
	}

	public checkSpecification(params: any) {
		return params.row.HAS_ATTACHMENT === 'Y';
	}

	public isIssue(params: any) {
		return params.row.TYPE === 'I';
	}

	public isIssueHasAction(params: any) {
		return params.row.TYPE === 'I' && params.row.HAS_ACTION === 'Y';
	}

	public async rowClickGridDemandManagement(params: any) {
		LoadingService.show();
		this.callDescription = params.row.REFERENCE;
		this.publicComment = false;
		this.showDemandManagementForm = true;
		this.callRelationship = params.row.CALL_RELATIONSHIP;
		this.callType = params.row.TYPE;
		this.callID = params.row.ID;
		this.callIsClosed = params.row.CLOSED;

		try {
			const callData = await Http.get('call', { params: { CALL_RELATIONSHIP: this.callRelationship, CALL_TYPE: this.callType } });
			LoadingService.hide();
			const tabMainFormDemandManagement = { component: null };
			if (params.row.TYPE === 'I') {
				this.showFormEditIssue = true;
				this.typeIsI = true;
				this.typeIsS = false;
				await this.getSelectsEditI(callData);
				tabMainFormDemandManagement.component = Metadata.getInstance<any>('tabMainFormDemandManagement');
				this.onCreatedIterableComments(tabMainFormDemandManagement);
			} else {
				this.showDemandManagementForm = true;
				this.typeIsI = false;
				this.typeIsS = true;
				await this.getSelectsEditS(callData);
				tabMainFormDemandManagement.component = Metadata.getInstance<any>('tabMainFormDemandManagementS');
				this.onCreatedIterableCommentsSoli(tabMainFormDemandManagement);
			}
		} catch (e) {
			LoadingService.hide();
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public getColumnStyle(params: any) {
		return params.row.ALREADY_READ_NAME !== 'Sim' ? 'pm-grid-column-bold' : '';
    }

    public async clickActionVisualizeCalls() {
		LoadingService.show();
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		try {
			await Http.post('/visualizeCall', { SELECTED_ROWS: callGridInstance.selectedRows });
			LoadingService.hide();
			AlertServiceCustom.defaultSaveSuccessAlert();
			callGridInstance.selectedRows = [];
			callGridInstance.datasource.get();
		} catch (e) {
			LoadingService.hide();
			DialogServiceCustom.showErrorBackend(e);
		}
    }

    public get checkedAsVisualizedCallActionDisabled() {
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		return callGridInstance.selectedRows.length === 0;
	}

	public async getSelectsEditI(callData: any) {
		const currentCustomerInstance = await InstanceService.getInstanceZdSelect('customerEditI');
		const productInstance = InstanceService.getInstanceZdSelect('productEditI');
		const categoryInstance = InstanceService.getInstanceZdSelect('categoryEditI');
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEditI');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		const versionDevIInstance = InstanceService.getInstanceZdSelect('devVersionEditI');

		// Aplicar Filtros nos Selects
		productInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
		productInstance.datasource.filter.APPLICATION_TYPE = this.callType;
		categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
		screenNameInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
		screenNameInstance.datasource.filter.PRODUCT_ID = callData.data.data.PRODUCT;
		screenNameInstance.datasource.get();
		const productIdByVersion = callData.data.data.DEVOPS_ID ?? callData.data.data.DEVOPS_ID_PRODUCT;
		versionIInstance.datasource.filter.PRODUCT_ID = productIdByVersion;
		versionDevIInstance.datasource.filter.PRODUCT_ID = callData.data.data.PRODUCT;
		versionDevIInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;

		// Atribuir Valor aos selects
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementI');
		callFormInstance.value.customerIdEditI = callData.data.data.ID; // I S
		callFormInstance.value.teknisaIdEditI = callData.data.data.CALL_RELATIONSHIP; // I S
		callFormInstance.value.reportedByEditI = callData.data.data.REPORTED_BY; // I S
		callFormInstance.value.referenceEditI = callData.data.data.REFERENCE; // I S
		callFormInstance.value.categoryEditI = callData.data.data.CLASSIFICATION; // I S
		callFormInstance.value.labelsEditI = callData.data.data.CALL_LABELS; // I S

		callFormInstance.value.initialAppointmentDateEditI = callData.data.data.APPOINTMENT_DATE_INIT; // I S
		callFormInstance.value.appointmentDateEditI = callData.data.data.APPOINTMENT_DATE; // I S
		callFormInstance.value.descriptionEditI = callData.data.data.DESCRIPTION; // I S

		await currentCustomerInstance.setValue(callData.data.data.CDCLIENTE, false);
		this.clientEditINotSelected = false;
		await productInstance.setValue(callData.data.data.PRODUCT, false);
		this.productEditINotSelected = false;
		await screenNameInstance.setValue(callData.data.data.MENU_ITEM_ID, false); // I
		this.screenNameEditINotSelected = false;
		await versionIInstance.setValue(callData.data.data.MENU_VERSION_ID, false); // I
		this.versionEditINotSelected = false;

		if (callData.data.data.VERSION_DEV_ID !== 'Última versão') {
			await versionDevIInstance.setValue(callData.data.data.VERSION_DEV_ID, false);
		} else {
			callFormInstance.value.devVersionEditI = callData.data.data.VERSION_DEV;
		}
	}

	public async getSelectsEditS(callData: any) {
		// const reportedByInstance = InstanceService.getInstanceZdSelect('reportedByEdit');
		const currentCustomerInstance = await InstanceService.getInstanceZdSelect('customerEdit');
		const productInstance = InstanceService.getInstanceZdSelect('productEdit');
		const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const responsibleByEdit = InstanceService.getInstanceZdSelect('responsibleByEdit');

		categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';

		responsibleByEdit.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
		responsibleByEdit.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;

		// Atribuir Valor aos selects
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementS');
		callFormInstance.value.customerIdEdit = callData.data.data.ID; // I S
		callFormInstance.value.teknisaIdEdit = callData.data.data.CALL_RELATIONSHIP; // I S
		callFormInstance.value.reportedByEdit = callData.data.data.REPORTER_BY; // I S
		callFormInstance.value.responsibleByEdit = callData.data.data.CDOPERRESP; // I S
		callFormInstance.value.referenceEdit = callData.data.data.REFERENCE; // I S

		callFormInstance.value.categoryEdit = callData.data.data.CLASSIFICATION; // I S

		callFormInstance.value.initialAppointmentDateEdit = callData.data.data.APPOINTMENT_DATE_INIT; // I S
		callFormInstance.value.appointmentDateEdit = callData.data.data.APPOINTMENT_DATE; // I S
		callFormInstance.value.descriptionEdit = callData.data.data.DESCRIPTION; // I S

		await currentCustomerInstance.setValue(callData.data.data.CDCLIENTE, false); // I S
		this.clientNOTSelected = false;

		// field productEdit
		productInstance.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		productInstance.datasource.filter.APPLICATION_TYPE = this.callType;

		callFormInstance.value.labelsEdit = callData.data.data.CALL_LABELS; // I S

		useLicenseInstance.datasource.filter.CDCLIENTE = callData.data.data.CDCLIENTE;
		useLicenseInstance.setValue(callData.data.data.USE_LICENSE, false); // S

		await productInstance.setValue(callData.data.data.SGSISTEMA, false); // I S
		this.productNOTSelected = false;

		this.productTypeNOTSelected = false;

		if (productInstance.value && productInstance.value?.SGSISTEMA) {
			productTypeInstance.datasource.filter.PRODUCT_CODE = productInstance.value?.SGSISTEMA;
			await productTypeInstance.setValue(callData.data.data.PRODUCT_TYPE, false); // S
		} else {
			DialogServiceCustom.showWarning(Messages.WARNING_NOT_CONTRACTED_PRODUCT_BY_CLIENT);
		}

		if (productTypeInstance.value && productTypeInstance.selectedData?.CDARVPROD && productInstance.selectedData?.SGSISTEMA) {
			versionSInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.value;
			versionSInstance.datasource.filter.PRODUCT_CONCAT = `${productTypeInstance.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;
			await versionSInstance.setValue(callData.data.data.PRODUCT_VERSION, false); // S
		}

		if (productTypeInstance.selectedData?.ID && versionSInstance.value) {
			releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
			releaseInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.value;
			await releaseInstance.setValue(callData.data.data.PRODUCT_RELEASE, false); // S
			this.versionNOTSelected = false;
		}

		if (productTypeInstance.selectedData?.ID && versionSInstance.selectedData?.ID && releaseInstance.value) {
			patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
			patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.selectedData.ID;
			patchInstance.datasource.filter.PRODUCT_RELEASE = releaseInstance.value;
			patchInstance.setValue(callData.data.data.PRODUCT_PATCH, false); // S
		}

		this.releaseNOTSelected = false;
	}

	// END CALL GRID -----

	// START MODAL ADD CALL ----
	public addButtonDemandManagementGrid() {
		this.modalAddCall.show();
	}

	public onChangeClientAddCallModal(params: any) { // CLIENT INPUT ADD
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			const productInstance = InstanceService.getInstanceZdSelect('productAdd');
			productInstance.value = '';
			this.typeIsI = false;
			this.typeIsS = false;
			this.resetValidations();
			if (params.value) {
				this.resetInputtedValuesADD();
				productInstance.value = '';
				productInstance.datasource.filter.CDCLIENTE = params.component.value;
				try {
					await productInstance.datasource.get();
					this.clientNOTSelectedAdd = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.clientNOTSelectedAdd = true;
					this.productNOTSelectedAdd = true;
					this.screenNameNOTSelected = true;
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					this.removeIValidations();
					this.removeSValidations();
				}
			} else {
				this.clientNOTSelectedAdd = true;
				this.productNOTSelectedAdd = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesADD();
				this.removeIValidations();
				this.removeSValidations();
			}
		}, 350);
	}

	public onChangeProductAddCallModal(params: any) { // PRODUCT ADD
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			if (params.value) {
				this.resetInputtedValuesADD();
				const customerInstance = InstanceService.getInstanceZdSelect('customerAdd');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryAdd');
				if (params.component.selectValue.PLATAFORMA === 'MOBILE') {
					const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
					this.setIValidations();
					this.removeSValidations();
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
					screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value;
					try {
						await categoryInstance.datasource.get();
						await screenNameInstance.datasource.get();
						this.typeIsI = true;
						this.typeIsS = false;
						this.productNOTSelectedAdd = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.typeIsI = false;
						this.typeIsS = false;
						this.productNOTSelectedAdd = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
						this.removeIValidations();
						this.removeSValidations();
					}
				} else {
					const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
					const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
					this.setSValidations();
					this.removeIValidations();
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
					productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.value;
					useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					try {
						await categoryInstance.datasource.get();
						await productTypeInstance.datasource.get();
						await useLicenseInstance.datasource.get();
						this.typeIsI = false;
						this.typeIsS = true;
						this.productNOTSelectedAdd = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.typeIsI = false;
						this.typeIsS = false;
						this.productNOTSelectedAdd = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
						this.removeIValidations();
						this.removeSValidations();
					}
				}
			} else {
				this.typeIsI = false;
				this.typeIsS = false;
				this.productNOTSelectedAdd = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.removeIValidations();
				this.removeSValidations();
				this.resetInputtedValuesADD();
			}
		}, 350);
	}

	public resetInputtedValuesADD() { // I S
		const addCallFormInstance = InstanceService.getInstanceZdForm('formAddCall');
		addCallFormInstance.value.screenNameAdd = '';
		addCallFormInstance.value.versionIAdd = '';
		addCallFormInstance.value.productTypeAdd = '';
		addCallFormInstance.value.versionSAdd = '';
		addCallFormInstance.value.releaseAdd = '';
		addCallFormInstance.value.patchAdd = '';
		addCallFormInstance.value.useLicenseAdd = '';
	}

	public setIValidations() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
		screenNameInstance.addValidation('required');
		versionIInstance.addValidation('required');
	}

	public removeIValidations() {
		const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameAdd');
		const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
		screenNameInstance.removeValidation('required');
		versionIInstance.removeValidation('required');
	}

	public setSValidations() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
		const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
		productTypeInstance.addValidation('required');
		useLicenseInstance.addValidation('required');
		versionSInstance.addValidation('required');
		releaseInstance.addValidation('required');
		patchInstance.addValidation('required');
	}

	public removeSValidations() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseAdd');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
		const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
		productTypeInstance.removeValidation('required');
		useLicenseInstance.removeValidation('required');
		versionSInstance.removeValidation('required');
		releaseInstance.removeValidation('required');
		patchInstance.removeValidation('required');
	}

	public onChangeScreenNameAddCallModal(params: any) { // SCREEN NAME
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionIAdd');
			if (params.component.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
				try {
					await versionIInstance.datasource.get();
					this.screenNameNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.screenNameNOTSelected = true;
				}
			} else {
				this.screenNameNOTSelected = true;
				versionIInstance.value = '';
			}
		}, 350);
	}

	public onChangeProductTypeAddCallModal(params: any) { // PRODUCT TYPE
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productInstance = InstanceService.getInstanceZdSelect('productAdd');
				versionSInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
				versionSInstance.datasource.filter.PRODUCT_CONCAT = `${params.component.selectedData.CDARVPROD}${productInstance.selectedData.SGSISTEMA}`;
				try {
					const version = await versionSInstance.datasource.get();
					if (version.length === 1) {
						this.versionNOTSelected = false;
						versionSInstance.value = version[0].ID;
						releaseInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
						releaseInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
						const release = await releaseInstance.datasource.get();
						if (release.length === 1) {
							this.releaseNOTSelected = false;
							releaseInstance.value = release[0].ID;
							patchInstance.datasource.filter.PRODUCT_TYPE = params.component.value;
							patchInstance.datasource.filter.PRODUCT_VERSION = version[0].ID;
							patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
							await patchInstance.datasource.get();
						}
					}
					this.productTypeNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.productTypeNOTSelected = true;
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
				}
			} else {
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				versionSInstance.value = '';
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputVersionSAddCallModal(params: any) { // VERSION S (DELPHI)
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const releaseInstance = InstanceService.getInstanceZdSelect('releaseAdd');
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				releaseInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				releaseInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
				try {
					const release = await releaseInstance.datasource.get();
					if (release.length === 1) {
						releaseInstance.value = release[0].ID;
						this.releaseNOTSelected = false;
						patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
						patchInstance.datasource.filter.PRODUCT_VERSION = params.component.value;
						patchInstance.datasource.filter.PRODUCT_RELEASE = release[0].ID;
						await patchInstance.datasource.get();
					}
					this.versionNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.versionNOTSelected = true;
					this.releaseNOTSelected = true;
					releaseInstance.value = '';
					patchInstance.value = '';
				}
			} else {
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				releaseInstance.value = '';
				patchInstance.value = '';
			}
		}, 350);
	}

	public onInputReleaseAddCallModal(params: any) { // RELEASE
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidations();
			const patchInstance = InstanceService.getInstanceZdSelect('patchAdd');
			if (params.value) {
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
				patchInstance.datasource.filter.PRODUCT_TYPE = productTypeInstance.selectedData.ID;
				patchInstance.datasource.filter.PRODUCT_VERSION = versionSInstance.selectValue.ID;
				patchInstance.datasource.filter.PRODUCT_RELEASE = params.component.value;
				try {
					await patchInstance.datasource.get();
					this.releaseNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.releaseNOTSelected = true;
					patchInstance.value = '';
				}
			} else {
				this.releaseNOTSelected = true;
				patchInstance.value = '';
			}
		}, 350);
	}

	public async onMountedFormAddCall() {
		const reportedByAddField = InstanceService.getInstanceZdSelect('reportedByAdd');
		await reportedByAddField.setValue(this.userAuth.CDOPERADOR, false);
	}

	public async saveAddNewCall() { // SAVE
		const addCallFormInstance = InstanceService.getInstanceZdForm('formAddCall');
		const checkValidation = addCallFormInstance.validate();
		if (checkValidation) {
			this.isLoadingDefault = true;
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerAdd');
			LoadingService.show();
			if (this.typeIsI) {
				try {
					await Http.post('/productCallSave', {
						REFERENCE: addCallFormInstance.value.referenceAdd,
						CALL_LABELS: addCallFormInstance.value.labelsAdd,
						DESCRIPTION: addCallFormInstance.value.descriptionAdd,
						REPORTED_BY: addCallFormInstance.value.reportedByAdd,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'MOBILE',
						PROJECT_ID: addCallFormInstance.value.productAdd,
						SGSISTEMA: null,
						TYPE: 'I',
						PRODUCT_VERSION: '0',
						PRODUCT_RELEASE: '0',
						MENU_ITEM_ID: addCallFormInstance.value.screenNameAdd.ID,
						DEVOPS_ID: addCallFormInstance.value.screenNameAdd.NREXTERNAL_PRODUCT_ID,
						MENU_VERSION_ID: addCallFormInstance.value.versionIAdd,
						CLASSIFICATION: addCallFormInstance.value.categoryAdd,
						APPOINTMENT_DATE: addCallFormInstance.value.appointmentDateAdd,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.closeAddModalDemandManagementGrid();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			} else {
				const productInstance = InstanceService.getInstanceZdSelect('productAdd');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeAdd');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionSAdd');
				try {
					await Http.post('/productCallSave', {
						REFERENCE: addCallFormInstance.value.referenceAdd,
						PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
						MENU_ITEM: '',
						MENU_VERSION: '',
						NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
						CALL_LABELS: addCallFormInstance.value.labelsAdd,
						DESCRIPTION: addCallFormInstance.value.descriptionAdd,
						REPORTED_BY: addCallFormInstance.value.reportedByAdd,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'DELPHI',
						PROJECT_ID: addCallFormInstance.value.productAdd,
						SGSISTEMA: productInstance.selectedData.SGSISTEMA,
						TYPE: 'S',
						PRODUCT: versionSInstance.selectValue.CDPRODCOMP,
						PRODUCT_VERSION: addCallFormInstance.value.versionSAdd,
						PRODUCT_RELEASE: addCallFormInstance.value.releaseAdd,
						PRODUCT_TYPE: addCallFormInstance.value.productTypeAdd,
						CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
						CLASSIFICATION: addCallFormInstance.value.categoryAdd,
						PRODUCT_PATCH: addCallFormInstance.value.patchAdd,
						USE_LICENSE: addCallFormInstance.value.useLicenseAdd,
						APPOINTMENT_DATE: addCallFormInstance.value.appointmentDateAdd,
					});
					LoadingService.hide();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.closeAddModalDemandManagementGrid();
					await callGridInstance.datasource.get();
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
				}
			}
			LoadingService.hide();
			this.isLoadingDefault = false;
		}
	}

	public resetValidations() {
		const addCallInstance = InstanceService.getInstanceZdForm('formAddCall');
		addCallInstance.resetValidation();
	}

	public closeAddModalDemandManagementGrid() {
		this.modalAddCall.hide();
		this.typeIsI = false;
		this.typeIsS = false;
		this.clientNOTSelected = true;
		this.productNOTSelected = true;
		this.screenNameNOTSelected = true;
		this.productTypeNOTSelected = true;
		this.versionNOTSelected = true;
		this.releaseNOTSelected = true;
	}
	// END MODAL ADD CALL ----

	// START MODAL FORWARD CALL ---
	public currentCustomerForwardCall = '';

	public questionLabelForwardCall = '';

	public disableForwardTo = true;

	public lastCommentLabel = '';

	public forwardCallType = '';

	public forwardCallRelationship = '';

	public REPORTED_BY = '';

	public forwardCDFASTRAMDES = '';

	public forwardNRFASTRAMDES = '';

	public forwardTXACAOFATRAM = '';

	public get forwardToDisable() {
		if (this.disableForwardTo) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public async closeSoli() {
		const sendObject = {
			CDCLIENTE: this.currentCustomerForwardCall,
			CALL_RELATIONSHIP: this.forwardCallRelationship,
		};
		await Http.post('/closeSoliServ', sendObject);

		this.clearModalForwardCall();

		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		await callGridInstance.datasource.get();
	}

	public async confirmCloseSoli() {
		const message = Messages.ALERT_CLOSED_CALL_RICH_CLIENT;
		const btnYes = [{
			name: Utilities.getNameDynamic('yes'),
			component: 'ZdButton',
			label: 'Sim',
			outline: false,
			events: {
				click: (() => {
					this.closeSoli();
					DialogService.hide();
				}),
			},
		}];
		DialogServiceCustom.confirmMessage(message, btnYes);
	}

	public async clickOpenForwardCall(params: any) {
		params.event.stopPropagation();
		this.currentCustomerForwardCall = params.row.CDCLIENTE;
		this.forwardCallType = params.row.TYPE;
		this.forwardCallRelationship = params.row.CALL_RELATIONSHIP;
		this.REPORTED_BY = params.row.REPORTED_BY;
		const closeCall = params.row.HAS_ACTION === 'C';
		if (closeCall) {
			await this.confirmCloseSoli();
		} else {
			await this.openModalFornwardSoli();
		}
	}

	public async clickOpenSpecification(params: any) {
		params.event.stopPropagation();
		this.callType = params.row.TYPE;
		this.callRelationship = params.row.CALL_RELATIONSHIP;
		if (this.callType === 'S') {
			await this.modalListSpecification.show();
			const listSpecificationInstance = InstanceService.getInstanceTekGrid('gridListSpecificationIssue');
			listSpecificationInstance.datasource.filter.CALL_TYPE = params.row.TYPE;
			listSpecificationInstance.datasource.filter.CALL_RELATIONSHIP = params.row.CALL_RELATIONSHIP;
			await listSpecificationInstance.datasource.get();
			if (listSpecificationInstance.datasource.data.length === 1) {
				this.rowSpec = { row: listSpecificationInstance.datasource.data[0] };
			}
		} else {
			await this.modalListSpecificationIssue.show();
			const result = await Http.post('/findAllSpecification', {
				page: 1,
				limit: 5000,
				order: ['CREATED_AT.desc'],
				0: 'CALL_TYPE',
				1: 'CALL_RELATIONSHIP',
				CALL_TYPE: this.callType,
				CALL_RELATIONSHIP: this.callRelationship,
			});
			console.log(result);
			if (result.data.data[0]) {
				const row = result.data.data[0];
				this.rowSpec = { row };
			}
		}
	}

	public async getReportSM() {
		LoadingService.show();
		try {
			this.baseUrl = Config.env.endPoint;
			let downloadUri = '';
			if (this.callType === 'S') {
				await Http.post('/soliSMBudgetValidate', {
					CALL_RELATIONSHIP: this.callRelationship,
				});
				downloadUri = `${this.baseUrl}/linkDownloadBudgetSmSoliserv?NRSOLISERV=${this.callRelationship}`;
			} else {
				downloadUri = `${this.baseUrl}/linkDownloadBudgetSm?ISSUE_ID=${this.callRelationship}`;
			}
			window.open(downloadUri);
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public isVisibleGridListSpecificationIssue() {
		return this.callType === 'S';
	}

	public async getReportSpeCli() {
		LoadingService.show();
		try {
			this.baseUrl = Config.env.endPoint;
			let downloadUri = '';
			if (this.callType === 'S') {
				await Http.post('/soliSpeCliValidate', {
					CALL_RELATIONSHIP: this.callRelationship,
				});
				downloadUri = `${this.baseUrl}/openPageRequisite?NRSOLISERV=${this.callRelationship}`;
			} else {
				downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${this.rowSpec.row.ID}`;
			}
			window.open(downloadUri);
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	private async openModalFornwardSoli() {
		await this.modalForwardCall.show();
		const questionSelectInstance = InstanceService.getInstanceZdSelect('questionSelectForwardCallModal');
		questionSelectInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
		questionSelectInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
		questionSelectInstance.datasource.filter.LANG = 'pt';
		try {
			const questionsTest = await questionSelectInstance.datasource.get();
			this.questionLabelForwardCall = `${questionsTest[0].TXPERGFATRAM}*`;
			const sendObject = {
				params: {
					CALL_TYPE: this.forwardCallType,
					CALL_RELATIONSHIP: this.forwardCallRelationship,
				},
			};
			const lastComment = await Http.get('/callComments', sendObject);
			if (lastComment.data.data.length > 0) {
				this.lastCommentLabel = lastComment.data.data[0].DESCRIPTION;
			} else {
				this.lastCommentLabel = 'Não possui cometário!';
			}
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
			this.closeModalForwardCall();
		}
	}

	public async onInputQuestionForwardCall(params: any) {
		const forwardToInstance = InstanceService.getInstanceZdSelect('forwardToForwardCallModal');
		if (params.value) {
			this.resetValidationsForwardCall();
			forwardToInstance.datasource.filter.CDCLIENTE = this.currentCustomerForwardCall;
			forwardToInstance.datasource.filter.CALL_TYPE = this.forwardCallType;
			forwardToInstance.datasource.filter.CALL_RELATIONSHIP = this.forwardCallRelationship;
			if (!params.component.selectedData.NRFASTRAMDES) {
				params.component.selectedData = Utilities.getRowByValueINSelect(params.value, params.component);
			}
			this.forwardTXACAOFATRAM = params.component.selectedData.TXACAOFATRAM;
			this.forwardCDFASTRAMDES = params.component.selectedData.CDFASTRAMDES;
			this.forwardNRFASTRAMDES = params.component.selectedData.NRFASTRAMDES;
			forwardToInstance.datasource.filter.CDFASTRAMDES = this.forwardCDFASTRAMDES;
			forwardToInstance.datasource.filter.NRFASTRAMDES = this.forwardNRFASTRAMDES;
			try {
				if (!forwardToInstance.value) {
					forwardToInstance.value = this.REPORTED_BY;
				}
				const whoFowardResponse = await forwardToInstance.datasource.get();
				if (whoFowardResponse.length === 1) {
					forwardToInstance.value = whoFowardResponse[0].CDOPERADOR;
					this.disableForwardTo = true;
				} else {
					this.disableForwardTo = false;
				}
			} catch (e) {
				DialogServiceCustom.showErrorBackend(e);
				this.disableForwardTo = true;
			}
		} else {
			this.disableForwardTo = true;
		}
	}

	public validadeCdfastramdes() {
		const aux = !!this.forwardCDFASTRAMDES;
		if (!aux) {
			AlertServiceCustom.customErrorAlert(Messages.ERROR_CDFASE);
		}
		return aux;
	}

	public async saveForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		const checkValidation = forwardCallInstance.validate();
		if (checkValidation && this.validadeCdfastramdes()) {
			const row = {
				CALL_RELATIONSHIP: this.forwardCallRelationship,
				CALL_TYPE: this.forwardCallType,
				CDFASTRAMDES: this.forwardCDFASTRAMDES,
				CDOPERADOR: forwardCallInstance.value.forwardToForwardCallModal,
				COMMENT: forwardCallInstance.value.observationForwardCallModal,
				DOCUMENT: <any>[],
				NRFASTRAMDES: this.forwardNRFASTRAMDES,
				TXACAOFATRAM: this.forwardTXACAOFATRAM,
				requestType: 'DataSet',
			};
			this.isLoadingDefault = true;
			const fileInput: any = Metadata.getInstance('fileInputForwardCallModal');
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;
				const checkFile = fileInput.value.map((file: any) => {
					if (file.size < 500000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});
				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							row.DOCUMENT = fileArray;
							await Http.post('/sendCallGep', row);
							this.closeModalForwardCall();
							AlertServiceCustom.defaultSaveSuccessAlert();
							this.isLoadingDefault = false;
						} catch (e) {
							DialogServiceCustom.showErrorBackend(e);
							this.isLoadingDefault = false;
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					this.isLoadingDefault = false;
				}
			} else {
				try {
					await Http.post('/sendCallGep', row);
					this.closeModalForwardCall();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.isLoadingDefault = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.isLoadingDefault = false;
				}
			}
        }
	}

	public resetValidationsForwardCall() {
		const forwardCallInstance = InstanceService.getInstanceZdForm('formForwardCallModal');
		forwardCallInstance.resetValidation();
	}

	public closeModalForwardCall() {
		this.modalForwardCall.hide();
		this.clearModalForwardCall();
	}

	public closeModalListSpecification() {
		this.modalListSpecification.hide();
	}

	public closeModalListSpecificationIssue() {
		this.modalListSpecificationIssue.hide();
	}

	public clearModalForwardCall() {
		this.currentCustomerForwardCall = '';
		this.questionLabelForwardCall = '';
		this.disableForwardTo = true;
		this.lastCommentLabel = '';
		this.forwardCallType = '';
		this.forwardCallRelationship = '';
		this.forwardCDFASTRAMDES = '';
		this.forwardNRFASTRAMDES = '';
		this.forwardTXACAOFATRAM = '';
	}
	// END MODAL FORWARD CALL ---

	// START MODAL ISSUE ACTION ---

	private issueActions = [];

	private listIssueActions: any = [];

	private currentIssueAction: any = {};

	private currentIssue: any = {};

	public isVisibleIssueActions = true;

	private getIssueIdByRow(concatenatedIssueId: string) {
		return concatenatedIssueId.replace('I - ', '');
	}

	public async clickIssueActions(params: any) {
		params.event.stopPropagation();
		this.currentIssue = params.row;
		try {
			const issueId = params.row.CALL_RELATIONSHIP;
			LoadingService.show();
			const { data } = await Http.get('/findActionsOfExits', { params: { issueId } });
			if (data.length) {
				await this.openModalIssueActions(data);
			} else {
				DialogServiceCustom.show({
					name: Utilities.getNameDynamic('notIssueActions'),
					title: 'Informação!',
					text: Messages.INFO_NOT_ACTION_ISSUE,
				});
			}
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
		}
	}

	public async closeModalIssueActions() {
		this.modalIssueActions.hide();
		this.modalIssueActions.destroy();
		this.listIssueActions = [];
	}

	public async closeIssue() {
		const ISSUE_ID = [this.currentIssue.CALL_RELATIONSHIP];
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		const data = { ISSUE_ID };
		LoadingService.show();
		await Http.post('/closeIssue', data);
		LoadingService.hide();
		AlertServiceCustom.defaultSaveSuccessAlert();
		this.isVisibleIssueActions = true;
		await this.closeModalIssueActions();
		await callGridInstance.datasource.get();
	}

	public clickItemIssueActionsFinalize(param: any) {
		this.currentIssueAction = this.getItemIssueActions(param.component.name);
		this.isVisibleIssueActions = true;
		const message = Messages.ALERT_CLOSED_CALL_RICH_CLIENT;
		const btnYes = [{
			name: Utilities.getNameDynamic('yes'),
			component: 'ZdButton',
			label: 'Sim',
			outline: false,
			events: {
				click: (() => {
					DialogService.hide();
					this.closeIssue();
				}),
			},
		}];
		DialogServiceCustom.confirmMessage(message, btnYes);
	}

	public clickItemIssueActions(param: any) {
		this.currentIssueAction = this.getItemIssueActions(param.component.name);
		this.isVisibleIssueActions = false;

		const formOutCommentModal = this.getFormOutCommentModal();
		formOutCommentModal.value.commentOutCommentModal = this.currentIssueAction.COMMENT_DESCRIPTION;

		const fileInputOutCommentModal = Metadata.getInstance<FileInput>('fileInputOutCommentModal');
		if (this.currentIssueAction.ATTACHMENT_REQUIRED === 'Y') {
			fileInputOutCommentModal.addValidation('required');
		} else {
			fileInputOutCommentModal.removeValidation('required');
		}
	}

	private async openModalIssueActions(data: any) {
		this.modalIssueActions = await ModalService.createFromJson('demandManagement/modal/issueActionsModal', true);
		await this.modalIssueActions.show();
		this.issueActions = data;
		this.createListByData();
		const listIssueActions = Metadata.getInstance<ZdList>('listIssueActions');
		listIssueActions.items = this.listIssueActions;
	}

	private getItemIssueActions(id: any) {
		return this.issueActions.filter((row: any) => row.ID === id)[0];
	}

	private createListByData() {
		this.issueActions.forEach((row: any) => {
			let item = {};
			let events = '{{DemandManagementController.clickItemIssueActions}}';
			if (row.SPECIAL_FUNCTION === 'F') {
				events = '{{DemandManagementController.clickItemIssueActionsFinalize}}';
			}

			item = {
				name: row.ID,
				component: 'ZdListItem',
				title: row.ACTION_LABEL,
				events: {
					click: events,
				},
			};
			this.listIssueActions.push(item);
		});
	}

	// Out Coment

	private getFormOutCommentModal() {
		return InstanceService.getInstanceZdForm('formOutCommentModal');
	}

	public backOutCommentModal() {
		const formOutCommentModal = this.getFormOutCommentModal();
		formOutCommentModal.value.commentOutCommentModal = '';
		formOutCommentModal.value.fileInputOutCommentModal = [];
		formOutCommentModal.resetValidation();
		this.isVisibleIssueActions = true;
	}

	public async saveOutCommentModal() {
		const formOutCommentModal = this.getFormOutCommentModal();
		if (formOutCommentModal.validate()) {
			const fileInput: any = Metadata.getInstance('fileInputOutCommentModal');
			const data: any = { ...this.currentIssueAction, ...formOutCommentModal.value };
			data.DESCRIPTION = data.commentOutCommentModal;
			data.ISSUE_ID = this.currentIssue.CALL_RELATIONSHIP;
			LoadingService.show();
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;

				const checkFile = fileInput.value.map((file: any) => {
					if ((file.type === 'application/msword'
						|| file.type === 'image/jpeg'
						|| file.type === 'image/png'
						|| file.type === 'application/pdf'
						|| file.type === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
						|| file.type === 'application/vnd.ms-excel.sheet.macroenabled.12'
						|| file.type === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
						|| file.type === 'application/vnd.ms-excel'
						|| file.type === 'text/plain'
						|| file.type === 'application/zip')
						&& file.size < 50000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});

				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							data.DOCUMENT = fileArray;
							await this.sendOutComment(data);
							AlertServiceCustom.defaultSaveSuccessAlert();
						} catch (e) {
							LoadingService.hide();
							DialogServiceCustom.showErrorBackend(e);
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					LoadingService.hide();
				}
			} else {
				try {
					await this.sendOutComment(data);
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			}
		}
	}

	private async sendOutComment(data: any) {
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		await Http.post('/sendOutComment', data);
		LoadingService.hide();
		AlertServiceCustom.defaultSaveSuccessAlert();
		this.isVisibleIssueActions = true;
		await this.closeModalIssueActions();
		await callGridInstance.datasource.get();
	}

	// END MODAL ISSUE ACTION ---

	// START CALL TAB ----
	public setIValidationsEDIT() {
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		versionIInstance.addValidation('required');
	}

	public removeIValidationsEDIT() {
		const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
		versionIInstance.removeValidation('required');
	}

	public setSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.addValidation('required');
		useLicenseInstance.addValidation('required');
		versionSInstance.addValidation('required');
		releaseInstance.addValidation('required');
		patchInstance.addValidation('required');
	}

	public removeSValidationsEDIT() {
		const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
		const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
		const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
		const releaseInstance = InstanceService.getInstanceZdSelect('releaseEdit');
		const patchInstance = InstanceService.getInstanceZdSelect('patchEdit');
		productTypeInstance.removeValidation('required');
		useLicenseInstance.removeValidation('required');
		versionSInstance.removeValidation('required');
		releaseInstance.removeValidation('required');
		patchInstance.removeValidation('required');
	}

	// Ref - I - S
	public resetValidationsEDIT() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.resetValidation();
	}

	// Ref - I
	public resetInputtedValuesEDIT() { // I S
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEdit = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEdit = '';
		callFormInstance.value.patchEdit = '';
		callFormInstance.value.useLicenseEdit = '';
	}

	// Ref - I
	public resetInputtedValuesByProductEditField() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		if (this.typeIsI) {
			callFormInstance.value.screenNameEdit = '';
			callFormInstance.value.versionEditI = '';
			callFormInstance.value.categoryEdit = '';
			callFormInstance.value.devVersionEdit = '';
		}
	}

	// Ref - I - S
	public onChangeProductEditCallModal(params: any) {
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			if (params.value) {
				this.resetInputtedValuesByProductEditField();
				const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
				const categoryInstance = InstanceService.getInstanceZdSelect('categoryEdit');
				if (params.component.selectValue.PLATAFORMA === 'MOBILE') {
					const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'I';
					screenNameInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					screenNameInstance.datasource.filter.PRODUCT_ID = params.component.value.PRODUCT_FILTER_ID;
					try {
						await categoryInstance.datasource.get();
						await screenNameInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				} else {
					const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
					const useLicenseInstance = InstanceService.getInstanceZdSelect('useLicenseEdit');
					categoryInstance.datasource.filter.APPLICATION_TYPE = 'S';
					productTypeInstance.datasource.filter.PRODUCT_CODE = params.component.datasource.data[0].SGSISTEMA;
					useLicenseInstance.datasource.filter.CDCLIENTE = customerInstance.selectedData.CDCLIENTE;
					try {
						await categoryInstance.datasource.get();
						await productTypeInstance.datasource.get();
						await useLicenseInstance.datasource.get();
						this.productNOTSelected = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.productNOTSelected = true;
						this.screenNameNOTSelected = true;
						this.productTypeNOTSelected = true;
						this.versionNOTSelected = true;
						this.releaseNOTSelected = true;
					}
				}
			} else {
				this.productNOTSelected = true;
				this.screenNameNOTSelected = true;
				this.productTypeNOTSelected = true;
				this.versionNOTSelected = true;
				this.releaseNOTSelected = true;
				this.resetInputtedValuesEDIT();
			}
		}, 350);
	}

	// Ref - I
	public onChangeScreenNameEditCallModal(params: any) { // SCREEN NAME EDIT
		clearTimeout(this.defaultTimeoutTimer);
		this.defaultTimeoutTimer = setTimeout(async () => {
			this.resetValidationsEDIT();
			const versionIInstance = InstanceService.getInstanceZdSelect('versionEditI');
			if (params.value) {
				versionIInstance.datasource.filter.PRODUCT_ID = params.component.value.NREXTERNAL_PRODUCT_ID;
				try {
					await versionIInstance.datasource.get();
					this.screenNameNOTSelected = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.screenNameNOTSelected = true;
				}
			} else {
				this.screenNameNOTSelected = true;
				versionIInstance.value = '';
			}
		}, 350);
	}

	// Ref - I
	public async saveEditCall() { // SAVE EDIT
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagement');
		const checkValidation = callFormInstance.validate();
		if (checkValidation) {
			this.isLoadingDefault = true;
			const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
			const customerInstance = InstanceService.getInstanceZdSelect('customerEdit');
			LoadingService.show();
			if (this.typeIsI) {
				const screenNameInstance = InstanceService.getInstanceZdSelect('screenNameEdit');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'MOBILE',
						PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
						SGSISTEMA: null,
						TYPE: 'I',
						PRODUCT_VERSION: '0',
						PRODUCT_RELEASE: '0',
						MENU_ITEM_ID: screenNameInstance.selectedData.ID,
						DEVOPS_ID: screenNameInstance.selectedData.NREXTERNAL_PRODUCT_ID,
						MENU_VERSION_ID: callFormInstance.value.versionEditI,
						VERSION_DEV_ID: callFormInstance.value.devVersionEditI,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			} else {
				const productInstance = InstanceService.getInstanceZdSelect('productEdit');
				const productTypeInstance = InstanceService.getInstanceZdSelect('productTypeEdit');
				const versionSInstance = InstanceService.getInstanceZdSelect('versionEditS');
				try {
					await Http.post('/productCallSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						ID: this.callID,
						REFERENCE: callFormInstance.value.referenceEdit,
						PRODUCT_TYPE_NAME: productTypeInstance.selectedData.LABEL,
						MENU_ITEM: '',
						MENU_VERSION: '',
						NMRAZSOCCLIE: customerInstance.selectedData.NMRAZSOCCLIE,
						CALL_LABELS: callFormInstance.value.labelsEdit,
						DESCRIPTION: callFormInstance.value.descriptionEdit,
						REPORTED_BY: callFormInstance.value.reportedByEdit,
						CDCLIENTE: customerInstance.selectedData.CDCLIENTE,
						PLATAFORMA: 'DELPHI',
						PROJECT_ID: callFormInstance.value.productEdit.PROJECT_ID,
						SGSISTEMA: productInstance.selectedData.SGSISTEMA,
						TYPE: 'S',
						PRODUCT: versionSInstance.selectValue.CDPRODCOMP,
						PRODUCT_VERSION: callFormInstance.value.versionEditS,
						PRODUCT_RELEASE: callFormInstance.value.releaseEdit,
						PRODUCT_TYPE: callFormInstance.value.productTypeEdit,
						CDARVPROD: productTypeInstance.selectedData.CDARVPROD,
						CLASSIFICATION: callFormInstance.value.categoryEdit,
						PRODUCT_PATCH: callFormInstance.value.patchEdit,
						USE_LICENSE: callFormInstance.value.useLicenseEdit,
						APPOINTMENT_DATE: callFormInstance.value.appointmentDateEdit,
					});
					AlertServiceCustom.defaultSaveSuccessAlert();
					await callGridInstance.datasource.get();
				} catch (e) {
					LoadingService.hide();
					DialogServiceCustom.showErrorBackend(e);
				}
			}
			this.isLoadingDefault = false;
			LoadingService.hide();
		} else {
			DialogServiceCustom.messageErrorValidate();
		}
	}

	public clickBackCardFormDemandManagement() {
		const callGridInstance = InstanceService.getInstanceTekGrid('gridDemandManagement');
		callGridInstance.datasource.get();
		this.showDemandManagementForm = false;
		this.showFormEditIssue = false;
		// Call Edit
		this.typeIsI = false;
		this.typeIsS = false;
		this.clientNOTSelected = true;
		this.productNOTSelected = true;
		this.screenNameNOTSelected = true;
		this.productTypeNOTSelected = true;
		this.versionNOTSelected = true;
		this.releaseNOTSelected = true;
		this.resetAllValues();
		// Comment
		const tabsInstance = Metadata.getInstance<ITabsBeforeChangeEventParam>('tabMainFormDemandManagement');
		tabsInstance.activeTab = 0;
	}

	public resetAllValues() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementI');
		callFormInstance.value.customerIdEditI = '';
		callFormInstance.value.teknisaIdEditI = '';
		callFormInstance.value.reportedByEditI = '';
		callFormInstance.value.referenceEditI = '';
		callFormInstance.value.customerEditI = '';
		callFormInstance.value.productEditI = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEditI = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEditI = '';
		callFormInstance.value.patchEditI = '';
		callFormInstance.value.useLicenseEditI = '';
		callFormInstance.value.initialAppointmentDateEditI = '';
		callFormInstance.value.appointmentDateEditI = '';
		callFormInstance.value.labelsEditI = '';
		callFormInstance.value.descriptionEditI = '';
	}

	public resetAllValuesFormEditIssue() {
		const callFormInstance = InstanceService.getInstanceZdForm('formDemandManagementI');
		callFormInstance.value.customerIdEditI = '';
		callFormInstance.value.teknisaIdEditI = '';
		callFormInstance.value.reportedByEditI = '';
		callFormInstance.value.referenceEditI = '';
		callFormInstance.value.customerEditI = '';
		callFormInstance.value.productEditI = '';
		callFormInstance.value.categoryEditI = '';
		callFormInstance.value.screenNameEditI = '';
		callFormInstance.value.versionEditI = '';
		callFormInstance.value.productTypeEditI = '';
		callFormInstance.value.versionEditS = '';
		callFormInstance.value.releaseEditI = '';
		callFormInstance.value.patchEditI = '';
		callFormInstance.value.useLicenseEditI = '';
		callFormInstance.value.initialAppointmentDateEditI = '';
		callFormInstance.value.appointmentDateEditI = '';
		callFormInstance.value.labelsEditI = '';
		callFormInstance.value.descriptionEditI = '';
	}
	// END CALL TAB ------

	// START COMMENTS TAB --
	public onCreatedIterableComments(params: any) {
		const tab = params.component.tabs[params.component.activeTab];
		setTimeout(async () => {
			if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
				await tab.events.change(params);
			}
		}, 100);
		this.eventsChangeTabMainFormDemandManagementS(params);
	}

	public async updateIterableDocuments() {
		// DOCUMENTS TAB
		let gridAttachments: any = null;

		if (this.callType === 'I') {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsI');
		} else {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsS');
		}

		gridAttachments.datasource.filter.CALL_TYPE = this.callType;
		gridAttachments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		try {
			await gridAttachments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public onCreatedIterableCommentsSoli(params: any) {
		const tab = params.component.tabs[params.component.activeTab];
		setTimeout(async () => {
			if (tab.events && tab.events.change && typeof tab.events.change === 'function') {
				await tab.events.change(params);
			}
		}, 100);
	}

	public updateBudgetSoli() {

	}

	public async changeGridAttachmentsS() {
		let gridAttachments: any = null;

		if (this.callType === 'I') {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsI');
		} else {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsS');
		}

		gridAttachments.datasource.filter.CALL_TYPE = this.callType;
		gridAttachments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		try {
			await gridAttachments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}

	public async eventsChangeTabMainFormDemandManagementS(params: any) {
		const activeTab = params.component.tabs[params.component.activeTab];
		if (activeTab.name === 'tabFormActivitiesDemandManagement') {
			setTimeout(async () => {
				await ActivitiesDemandManagement.reloadActivitiesDemandManagement(this.callRelationship);
			}, 500);
		}
	}

	public onDestroyButtonAddComments() {
		this.modalAddComments?.destroy();
	}

	public clickIterableComments(params: any) {
		console.log(params);
	}

	public onMountedBuildFileIterableData(params: any) {
		const id = params.component.name.split('-')[1];
		let iterableCommentsInstance: any = null;

		if (this.callType === 'I') {
			iterableCommentsInstance = Metadata.getInstance<IterableComponentRender>('iterableCommentsI');
		} else {
			iterableCommentsInstance = Metadata.getInstance<IterableComponentRender>('iterableCommentsS');
		}

		const rowData = iterableCommentsInstance.datasource.data.find((row: { ID: any; }) => row.ID === id);
		if (rowData) {
			if (rowData.ATTACHMENT.length > 0) {
				params.component.datasource.data = rowData.ATTACHMENT;
			}
		}
	}

	public downloadFileComment(params: any) {
		const fileId = params.element.name.split('-')[1];
		const commentId = params.element.name.split('-')[2];
		const fileInstance = Metadata.getInstance<IterableComponentRender>(`iterableFiles-${commentId}`);
		const fileSelected = fileInstance.datasource.data.find((file) => file.ID === fileId);
		if (fileSelected) {
			if (this.callType === 'I') {
				const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${fileSelected.ID}`;
				window.open(downloadUri);
			} else {
				const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILENAME=${fileSelected.FILENAME}`;
				window.open(downloadUri);
			}
		}
	}

	public async updateIterableComments() {
		let iterableComments: any = null;

		if (this.callType === 'I') {
			iterableComments = Metadata.getInstance<IterableComponentRender>('iterableCommentsI');
		} else {
			iterableComments = Metadata.getInstance<IterableComponentRender>('iterableCommentsS');
		}

		iterableComments.datasource.filter.CALL_TYPE = this.callType;
		iterableComments.datasource.filter.CALL_RELATIONSHIP = this.callRelationship;
		try {
			await iterableComments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END COMMENTS TAB ----

	// START COMMENTS MODAL ----
	public defaultPublicCommentValue = 'N';

	public async addCommentButton() {
		this.modalAddComments = await ModalService.createFromJson('demandManagement/modal/addCommentsDemandManagementModal', true);
		this.modalAddComments.show();
		if (this.publicComment) {
			this.defaultPublicCommentValue = 'Y';
		}
	}

	public closeAddCommentsModal() {
		this.modalAddComments.hide();
		this.defaultPublicCommentValue = 'N';
		this.modalAddComments?.destroy();
	}

	public get switchButtonDisabledAddCommentsModal() {
		if (this.publicComment) {
			return true;
		}
		return this.isLoadingDefault;
	}

	public async saveAddComment() {
		const addCommentInstance = InstanceService.getInstanceZdForm('formAddCommentsModal');
		const commentValidation = addCommentInstance.validate();
        if (commentValidation) {
			this.isLoadingDefault = true;
			const fileInput: any = Metadata.getInstance('fileInputAddCommentsModal');
			if (fileInput.value && fileInput.value.length > 0) {
				let mimeCheck = true;
				const checkFile = fileInput.value.map((file: any) => {
					if (file.size < 500000000) {
						return FileUpload.buildFileSendObject(file);
					}
					mimeCheck = false;
					return false;
				});
				if (mimeCheck) {
					Promise.all(checkFile).then(async (fileArray) => {
						try {
							await Http.post('/callAttachmentSave', {
								CALL_RELATIONSHIP: this.callRelationship,
								CALL_TYPE: this.callType,
								DESCRIPTION: addCommentInstance.value.commentTextAreaAddCommentsModal,
								PUBLIC: addCommentInstance.value.switchButtonAddCommentsModal,
								DOCUMENT: fileArray,
								requestType: 'DataSet',
							});
							this.updateIterableComments();
							this.closeAddCommentsModal();
							AlertServiceCustom.defaultSaveSuccessAlert();
							this.isLoadingDefault = false;
						} catch (e) {
							DialogServiceCustom.showErrorBackend(e);
							this.isLoadingDefault = false;
						}
					});
				} else {
					AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
					this.isLoadingDefault = false;
				}
			} else {
				try {
					await Http.post('/callAttachmentSave', {
						CALL_RELATIONSHIP: this.callRelationship,
						CALL_TYPE: this.callType,
						DESCRIPTION: addCommentInstance.value.commentTextAreaAddCommentsModal,
						PUBLIC: addCommentInstance.value.switchButtonAddCommentsModal,
						DOCUMENT: [],
						requestType: 'DataSet',
					});
					this.updateIterableComments();
					this.closeAddCommentsModal();
					AlertServiceCustom.defaultSaveSuccessAlert();
					this.isLoadingDefault = false;
				} catch (e) {
					DialogServiceCustom.showErrorBackend(e);
					this.isLoadingDefault = false;
				}
			}
        }
	}
	// END COMMENTS MODAL ----

	// START DOCUMENTS TAB ---
	public onDestroyGridAttachments() {
		this.modalUploadFile.destroy();
	}

	public async addClickGridAttachments() {
		this.modalUploadFile = await ModalService.createFromJson('demandManagement/modal/uploadFileDemandManagementModal', true);
		this.modalUploadFile.show();
	}

	public downloadFileGrid(params: any) {
		if (this.callType === 'I') {
			const downloadUri = `${this.baseUrl}/downLoadAttachment?ID=${params.row.ID}`;
			window.open(downloadUri);
		} else {
			const downloadUri = `${this.baseUrl}/downLoadAttachmentGEP?FILENAME=${params.row.FILENAME}`;
			window.open(downloadUri);
		}
	}

	public async deleteAttachmentsGrid(params: any) {
		params.event.preventDefault();
		const buttonYes = [
			{
				name: Utilities.getNameDynamic('yes-'),
				component: 'ZdButton',
				label: 'Sim',
				events: {
					click: (() => {
						DialogService.hide();
						this.doDeleteAttachmentsGrid(params);
					}),
				},
			},
		];
		DialogServiceCustom.confirmMessage(Messages.DELETE_DEFAULT, buttonYes);
	}

	public async doDeleteAttachmentsGrid(params: any) {
		LoadingService.show();
		try {
			await Http.post('deleteProductCallAttachments', params.component.selectedRows);
			params.component.selectedRows = [];
			await this.updateFileGrid();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
		LoadingService.hide();
	}

	public async updateFileGrid() {
		let gridAttachments: any = null;

		if (this.callType === 'I') {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsI');
		} else {
			gridAttachments = Metadata.getInstance<any>('gridAttachmentsS');
		}

		try {
			await gridAttachments.datasource.get();
		} catch (e) {
			DialogServiceCustom.showErrorBackend(e);
		}
	}
	// END DOCUMENTS TAB -----

	// START DOCUMENTS MODAL ---
	public async saveUploadFileModal() {
		this.isLoadingDefault = true;
		let mimeCheck = true;
        const fileInput: any = Metadata.getInstance('fileInputUploadFileDemandManagementModal');
        if (fileInput.value && fileInput.value.length > 0) {
			const checkFile = fileInput.value.map((file: any) => {
				if ((file.type === 'application/msword'
				|| file.type === 'image/jpeg'
				|| file.type === 'image/png'
				|| file.type === 'application/pdf'
				|| file.type === 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
				|| file.type === 'application/vnd.ms-excel.sheet.macroenabled.12'
				|| file.type === 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
				|| file.type === 'application/vnd.ms-excel'
				|| file.type === 'text/plain'
				|| file.type === 'application/zip')
				&& file.size < 50000000) {
					return FileUpload.buildFileSendObject(file);
				}
				mimeCheck = false;
				return false;
			});
			if (mimeCheck) {
				Promise.all(checkFile).then(async (fileArray) => {
					try {
						await Http.post('/callAttachmentSave', {
							CALL_RELATIONSHIP: this.callRelationship,
							CALL_TYPE: this.callType,
							DOCUMENT: fileArray,
							requestType: 'DataSet',
						});
						await this.updateFileGrid();
						this.closeUploadFileModal();
						AlertServiceCustom.defaultSaveSuccessAlert();
						this.isLoadingDefault = false;
					} catch (e) {
						DialogServiceCustom.showErrorBackend(e);
						this.isLoadingDefault = false;
					}
				});
			} else {
				AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_TYPE);
				this.isLoadingDefault = false;
			}
        } else {
			AlertServiceCustom.customErrorAlert(Messages.ERROR_FILE_EXPECTED);
			this.isLoadingDefault = false;
        }
	}

	public closeUploadFileModal() {
		this.modalUploadFile.hide();
		this.modalUploadFile.destroy();
	}

	public onChangeTeste(params: any) {
		console.log(params.component.value);
	}
	// END DOCUMENTS MODAL -----
}
